;;; sdl.scm - Simple SDL binding for Chicken
; Copyright (C) 2002-2004 Tony Garnock-Jones <tonyg@kcbbs.gen.nz>
;
; This library is free software; you can redistribute it and/or modify
; it under the terms of the GNU Library General Public License as
; published by the Free Software Foundation; either version 2 of the
; License, or (at your option) any later version.
;
; This library is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
; Library General Public License for more details.
;
; You should have received a copy of the GNU Library General Public
; License along with this library; if not, write to the Free
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
; USA

; ---------------------------------------------------------------------------

(declare
 (usual-integrations)

 ;; %%% export declaration should go here

 (fixnum)
 (foreign-declare #<<EOF

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_image.h"
#include "SDL_rotozoom.h"

unsigned int chickensdl_timer_scheme_callback(unsigned int interval,
						       int cbnum);

static Uint32 chickenSDL_timer_c_callback(Uint32 interval,
					  void *param)
{
  return chickensdl_timer_scheme_callback(interval, (int) param);
}

EOF
))

(require 'srfi-1)
(require 'srfi-13)
(require 'lolevel)

(require-for-syntax 'srfi-1)
(require-for-syntax 'srfi-13)

;---------------------------------------------------------------------------

(define *sdl-egg-version* '(0 2 1))

;---------------------------------------------------------------------------

(define-macro (--sdl-flags . strs)
  `(begin
     ,@(append-map (lambda (str)
		     (let* ((sym (string->symbol (string-downcase str)))
			    (psym (string->symbol (string-append "-" (symbol->string sym)))))
		       `((define-foreign-variable ,psym unsigned-integer ,str)
			 (define ,sym ,psym))))
		   strs)))

; Subsystem definitions, for sdl-init etc.
(--sdl-flags "SDL_INIT_TIMER"
	     "SDL_INIT_AUDIO"
	     "SDL_INIT_VIDEO"
	     "SDL_INIT_CDROM"
	     "SDL_INIT_JOYSTICK"
	     "SDL_INIT_EVERYTHING"
	     "SDL_INIT_NOPARACHUTE")

(--sdl-flags
	;; For sdl-creatergbsurface or sdl-setvideomode
	"SDL_SWSURFACE"
	"SDL_HWSURFACE"
	"SDL_ASYNCBLIT"
	;; For sdl-setvideomode
	"SDL_ANYFORMAT"
	"SDL_HWPALETTE"
	"SDL_DOUBLEBUF"
	"SDL_FULLSCREEN"
	"SDL_OPENGL"
	"SDL_OPENGLBLIT"
	"SDL_RESIZABLE"
	"SDL_NOFRAME"
	;; Read-only - internal
	"SDL_HWACCEL"
	"SDL_SRCCOLORKEY"
	"SDL_RLEACCELOK"
	"SDL_RLEACCEL"
	"SDL_SRCALPHA"
	"SDL_PREALLOC"
)

; For sdl-wm-grabinput
(--sdl-flags "SDL_GRAB_QUERY"
	     "SDL_GRAB_OFF"
	     "SDL_GRAB_ON")

(--sdl-flags
	"SDL_NOEVENT"			; Unused (do not remove) */
	"SDL_ACTIVEEVENT"		; Application loses/gains visibility
	"SDL_KEYDOWN"			; Keys pressed
	"SDL_KEYUP"			; Keys released
	"SDL_MOUSEMOTION"		; Mouse moved
	"SDL_MOUSEBUTTONDOWN"		; Mouse button pressed
	"SDL_MOUSEBUTTONUP"		; Mouse button released
	"SDL_JOYAXISMOTION"		; Joystick axis motion
	"SDL_JOYBALLMOTION"		; Joystick trackball motion
	"SDL_JOYHATMOTION"		; Joystick hat position change
	"SDL_JOYBUTTONDOWN"		; Joystick button pressed
	"SDL_JOYBUTTONUP"		; Joystick button released
	"SDL_QUIT"			; User-requested quit
	"SDL_SYSWMEVENT"		; System specific event
	"SDL_EVENT_RESERVEDA"		; Reserved for future use..
	"SDL_EVENT_RESERVEDB"		; Reserved for future use..
	"SDL_VIDEORESIZE"		; User resized video mode
	"SDL_VIDEOEXPOSE"		; Screen needs to be redrawn
	"SDL_EVENT_RESERVED2"		; Reserved for future use..
	"SDL_EVENT_RESERVED3"		; Reserved for future use..
	"SDL_EVENT_RESERVED4"		; Reserved for future use..
	"SDL_EVENT_RESERVED5"		; Reserved for future use..
	"SDL_EVENT_RESERVED6"		; Reserved for future use..
	"SDL_EVENT_RESERVED7"		; Reserved for future use..
	"SDL_USEREVENT"			; SDL_USEREVENT .. SDL_NUMEVENTS are for client use
	"SDL_NUMEVENTS"
)

(--sdl-flags
	"SDL_ACTIVEEVENTMASK"
	"SDL_KEYDOWNMASK"
	"SDL_KEYUPMASK"
	"SDL_MOUSEMOTIONMASK"
	"SDL_MOUSEBUTTONDOWNMASK"
	"SDL_MOUSEBUTTONUPMASK"
	"SDL_MOUSEEVENTMASK"
	"SDL_JOYAXISMOTIONMASK"
	"SDL_JOYBALLMOTIONMASK"
	"SDL_JOYHATMOTIONMASK"
	"SDL_JOYBUTTONDOWNMASK"
	"SDL_JOYBUTTONUPMASK"
	"SDL_JOYEVENTMASK"
	"SDL_VIDEORESIZEMASK"
	"SDL_VIDEOEXPOSEMASK"
	"SDL_QUITMASK"
	"SDL_SYSWMEVENTMASK"
	"SDL_ALLEVENTS"
)

; For sdl-eventstate
(--sdl-flags "SDL_QUERY"
	     "SDL_IGNORE"
	     "SDL_DISABLE"
	     "SDL_ENABLE")

;---------------------------------------------------------------------------

(define-foreign-variable sizeof-sdl-rect int "sizeof(SDL_Rect)")

(define-record sdl-rect buffer)

(define *sdl-rect-nil* (make-sdl-rect #f))

(let ((maker make-sdl-rect))
  (set! make-sdl-rect
	(lambda (x y w h)
	  (let ((r (maker (make-byte-vector sizeof-sdl-rect))))
	    (sdl-rect-x-set! r x)
	    (sdl-rect-y-set! r y)
	    (sdl-rect-w-set! r w)
	    (sdl-rect-h-set! r h)
	    r))))

(define-record-printer (sdl-rect s out)
  (for-each (lambda (x) (display x out))
	    (if (sdl-rect-buffer s)
		(list "#<sdl-rect "
		      (sdl-rect-x s)" "
		      (sdl-rect-y s)" "
		      (sdl-rect-w s)" "
		      (sdl-rect-h s)">")
		(list "#<sdl-rect-nil>"))))

(define (-sdl-unbox-rect e)
  (let ((p (##sys#make-pointer))
	(b (sdl-rect-buffer e)))
    (if b (##core#inline "C_pointer_to_block" p b))
    p))

(define-foreign-type SDL_Rect (pointer "SDL_Rect")
  -sdl-unbox-rect)

(define sdl-rect-x (foreign-lambda* short ((SDL_Rect c)) "return(c->x);"))
(define sdl-rect-y (foreign-lambda* short ((SDL_Rect c)) "return(c->y);"))
(define sdl-rect-w (foreign-lambda* unsigned-short ((SDL_Rect c)) "return(c->w);"))
(define sdl-rect-h (foreign-lambda* unsigned-short ((SDL_Rect c)) "return(c->h);"))

(define sdl-rect-x-set! (foreign-lambda* void ((SDL_Rect c) (short x)) "c->x = x;"))
(define sdl-rect-y-set! (foreign-lambda* void ((SDL_Rect c) (short y)) "c->y = y;"))
(define sdl-rect-w-set! (foreign-lambda* void ((SDL_Rect c) (unsigned-short w)) "c->w = w;"))
(define sdl-rect-h-set! (foreign-lambda* void ((SDL_Rect c) (unsigned-short h)) "c->h = h;"))

;---------------------------------------------------------------------------

(define-record sdl-pixel-format pointer)

(define-record-printer (sdl-pixel-format p out)
  (for-each (lambda (x) (display x out))
	    (list "#<sdl-pixel-format "(sdl-pixel-format-pointer p)">")))

(define-foreign-type SDL_PixelFormat (pointer "SDL_PixelFormat")
  sdl-pixel-format-pointer
  make-sdl-pixel-format)

;---------------------------------------------------------------------------

(define-record sdl-surface pointer)

(define-record-printer (sdl-surface s out)
  (for-each (lambda (x) (display x out))
	    (list "#<sdl-surface "(sdl-surface-pointer s)">")))

(define-foreign-type SDL_Surface (pointer "SDL_Surface")
  sdl-surface-pointer
  make-sdl-surface)

(define (sdl-surface-flags s)
  ((foreign-lambda* unsigned-integer ((SDL_Surface s))
		    "return(s->flags);") s))

(define (sdl-surface-pixel-format s)
  ((foreign-lambda* SDL_PixelFormat ((SDL_Surface s))
		    "return(s->format);") s))

(define (sdl-surface-width s)
  ((foreign-lambda* integer ((SDL_Surface s))
		    "return(s->w);") s))

(define (sdl-surface-height s)
  ((foreign-lambda* integer ((SDL_Surface s))
		    "return(s->h);") s))

(define (sdl-surface-pitch s)
  ((foreign-lambda* unsigned-short ((SDL_Surface s))
		    "return(s->pitch);") s))

;; Modifies its second argument.
(define sdl-get-clip-rect! (foreign-lambda void "SDL_GetClipRect" SDL_Surface SDL_Rect))

;; Modifies its first argument.
(define sdl-set-clip-rect! (foreign-lambda bool "SDL_SetClipRect" SDL_Surface SDL_Rect))

;---------------------------------------------------------------------------

;; NOTE: sdl-init does not work on MacOS X when called from a
;; dynamically-loaded extension. Something internal to Quartz seems to
;; get confused. You must call SDL_Init *directly* from your main
;; program - if your main program is written in Scheme, you need to
;; say something like:
;;
;; (declare (foreign-declare "#include <SDL.h>\n"))
;; (foreign-code "SDL_Init(SDL_INIT_EVERYTHING);")
;;
(define (sdl-init flags)
  (zero? ((foreign-lambda int "SDL_Init" unsigned-integer) flags)))

;; Note: guile-sdl names these 'sdl-init-subsystem' and
;; 'sdl-quit-subsystem', respectively.

(define (sdl-init-sub-system flags)
  (zero? ((foreign-lambda int "SDL_InitSubSystem" unsigned-integer) flags)))

(define (sdl-quit-sub-system)
  ((foreign-lambda void "SDL_QuitSubSystem" unsigned-integer) flags))

(define (sdl-quit)
  ((foreign-lambda void "SDL_Quit")))

(define (sdl-was-init flags)
  ((foreign-lambda unsigned-integer "SDL_WasInit" unsigned-integer) flags))

;---------------------------------------------------------------------------

(define (sdl-wm-set-caption title icon)
  ((foreign-lambda void "SDL_WM_SetCaption" c-string c-string) title icon))

(define (sdl-wm-get-caption-title)
  ((foreign-lambda* c-string ()
		    "char *t, *i;"
		    "SDL_WM_GetCaption(&t, &i);"
		    "return(t);")))

(define (sdl-wm-get-caption-icon)
  ((foreign-lambda* c-string ()
		    "char *t, *i;"
		    "SDL_WM_GetCaption(&t, &i);"
		    "return(i);")))

(define (sdl-wm-get-caption)
  (values (sdl-wm-getcaption-title)
	  (sdl-wm-getcaption-icon)))

(define (sdl-wm-set-icon icon mask)
  ((foreign-lambda void "SDL_WM_SetIcon" SDL_Surface byte-vector) icon mask))

(define (sdl-wm-iconify-window)
  (not (zero? ((foreign-lambda integer "SDL_WM_IconifyWindow")))))

(define (sdl-wm-toggle-full-screen surf)
  (not (zero? ((foreign-lambda integer "SDL_WM_ToggleFullScreen" SDL_Surface) surf))))

(define (sdl-wm-grab-input m)
  ((foreign-lambda integer "SDL_WM_GrabInput" integer) m))

;---------------------------------------------------------------------------

; Milliseconds.
(define sdl-get-ticks (foreign-lambda unsigned-integer "SDL_GetTicks"))
(define sdl-delay (foreign-lambda void "SDL_Delay" unsigned-integer))

(define -sdl-callbacks (make-hash-table))
(define -sdl-next-callback 0)

(define-external (chickensdl_timer_scheme_callback (unsigned-integer interval)
						   (integer cbnum))
  unsigned-integer
  (let ((cb (hash-table-ref -sdl-callbacks cbnum)))
    (if cb
	((car cb) interval cbnum)
	(error "chickensdl_timer_scheme_callback: Unknown SDL timer callback " cbnum))))

(define (sdl-add-timer interval callback)
  (let ((i -sdl-next-callback))
    (set! -sdl-next-callback (+ i 1))
    (let ((p ((foreign-lambda* c-pointer ((unsigned-integer interval)
					  (integer cbnum))
			       "return(SDL_AddTimer(interval,"
			       "                    chickenSDL_timer_c_callback,"
			       "                    (void *) cbnum));")
	      interval
	      i)))
      (if (or (not p)
	      (null-pointer? p))
	  #f
	  (begin
	    (hash-table-set! -sdl-callbacks i (cons callback p))
	    i)))))

(define (sdl-remove-timer cbnum)
  (let ((cb (hash-table-ref -sdl-callbacks cbnum)))
    (if cb
	(begin
	  (hash-table-remove! -sdl-callbacks cbnum)
	  ((foreign-lambda bool "SDL_RemoveTimer" pointer) (cdr cb)))
	(error "sdl-removetimer: Unknown SDL timer callback" cbnum))))

; sdl-set-timer not supported - obsoleted by sdl-addtimer/sdl-removetimer

;---------------------------------------------------------------------------

(define-foreign-variable sizeof-sdl-event int "sizeof(SDL_Event)")

(define-record sdl-event buffer)

(let ((maker make-sdl-event))
  (set! make-sdl-event
	(lambda ()
	  (let ((bv (make-byte-vector sizeof-sdl-event)))
	    (byte-vector-set! bv 0 sdl_noevent)
	    (maker bv)))))

(define-record-printer (sdl-event s out)
  (for-each (lambda (x) (display x out))
	    (list "#<sdl-event "(sdl-event-type s)">")))

(define (-sdl-unbox-event e)
  (let ((p (##sys#make-pointer)))
    (##core#inline "C_pointer_to_block" p (sdl-event-buffer e))
    p))

(define-foreign-type SDL_Event (pointer "SDL_Event")
  -sdl-unbox-event)

(define-foreign-type byte unsigned-int)

(define sdl-event-type (foreign-lambda* byte ((SDL_Event e)) "return(e->type);"))
(define sdl-event-type-set! (foreign-lambda* void ((SDL_Event e) (byte t)) "e->type = t;"))

(define-macro (--sdl-event-getter-setter name . rest)
  (let* ((strapp (lambda s (apply string-append
				  (map (lambda (x) (cond
						    ((symbol? x) (symbol->string x))
						    (else x)))
				       s))))
	 (symapp (lambda s (string->symbol (apply strapp s)))))
  `(begin
     (define (,(symapp "sdl-event-" name) e)
       (let ((t (sdl-event-type e)))
	 (cond
	  ,@(map (lambda (clause)
		   (apply (lambda (etype mem1 kind)
			    `((= t ,etype) ((foreign-lambda*
					     ,kind ((SDL_Event e))
					     ,(strapp "return(e->"mem1"."name");")) e)))
			  clause))
		 rest)
	  (else (error ,(string-append "sdl-event-" (symbol->string name)
				       ": cannot extract value from this type of event")
		       (sdl-event-type e))))))
     (define (,(symapp "set-sdl-event-" name "!") e v)
       (let ((t (sdl-event-type e)))
	 (cond
	  ,@(map (lambda (clause)
		   (apply (lambda (etype mem1 kind)
			    `((= t ,etype) ((foreign-lambda*
					     void ((SDL_Event e)
						   (,kind v))
					     ,(strapp "e->"mem1"."name"=v;")) e v)))
			  clause))
		 rest)
	  (else (error ,(string-append "set-sdl-event-" (symbol->string name) "!"
				       ": cannot update value for this type of event")
		       (sdl-event-type e)))))))))

(--sdl-event-getter-setter gain		(sdl_activeevent active byte))
(--sdl-event-getter-setter which	(sdl_keydown key byte)
					(sdl_keyup key byte)
					(sdl_mousemotion motion byte)
					(sdl_mousebuttondown button byte)
					(sdl_mousebuttonup button byte)
					(sdl_joyaxismotion jaxis byte)
					(sdl_joyballmotion jball byte)
					(sdl_joyhatmotion jhat byte)
					(sdl_joybuttondown jbutton byte)
					(sdl_joybuttonup jbutton byte))
(--sdl-event-getter-setter state	(sdl_activeevent active byte)
					(sdl_keydown key byte)
					(sdl_keyup key byte)
					(sdl_mousemotion motion byte)
					(sdl_mousebuttondown button byte)
					(sdl_mousebuttonup button byte)
					(sdl_joybuttondown jbutton byte)
					(sdl_joybuttonup jbutton byte))
(--sdl-event-getter-setter scancode	(sdl_keydown key.keysym byte)
					(sdl_keyup   key.keysym byte))
(--sdl-event-getter-setter sym		(sdl_keydown key.keysym integer)
					(sdl_keyup   key.keysym integer))
(--sdl-event-getter-setter mod		(sdl_keydown key.keysym integer)
					(sdl_keyup   key.keysym integer))
(--sdl-event-getter-setter unicode	(sdl_keydown key.keysym short)
					(sdl_keyup   key.keysym short))
(--sdl-event-getter-setter x		(sdl_mousemotion motion unsigned-short)
					(sdl_mousebuttondown button unsigned-short)
					(sdl_mousebuttonup button unsigned-short))
(--sdl-event-getter-setter y		(sdl_mousemotion motion unsigned-short)
					(sdl_mousebuttondown button unsigned-short)
					(sdl_mousebuttonup button unsigned-short))
(--sdl-event-getter-setter xrel		(sdl_mousemotion motion short)
					(sdl_joyballmotion jball short))
(--sdl-event-getter-setter yrel		(sdl_mousemotion motion short)
					(sdl_joyballmotion jball short))
(--sdl-event-getter-setter axis		(sdl_joyaxismotion jaxis byte))
(--sdl-event-getter-setter ball		(sdl_joyballmotion jball byte))
(--sdl-event-getter-setter hat		(sdl_joyhatmotion jhat byte))
(--sdl-event-getter-setter value	(sdl_joyaxismotion jaxis short)
					(sdl_joyhatmotion jhat byte))
(--sdl-event-getter-setter button	(sdl_mousebuttondown button byte)
					(sdl_mousebuttonup button byte)
					(sdl_joybuttondown jbutton byte)
					(sdl_joybuttonup jbutton byte))
(--sdl-event-getter-setter w		(sdl_videoresize resize integer))
(--sdl-event-getter-setter h		(sdl_videoresize resize integer))

(define sdl-pump-events (foreign-lambda void "SDL_PumpEvents"))

(define (sdl-poll-event! . e)
  (if (null? e)
      (not (zero? ((foreign-lambda int "SDL_PollEvent" pointer) #f)))
      (not (zero? ((foreign-lambda int "SDL_PollEvent" SDL_Event) (car e))))))

(define (sdl-wait-event! . e)
  (if (null? e)
      (not (zero? ((foreign-lambda int "SDL_WaitEvent" pointer) #f)))
      (not (zero? ((foreign-lambda int "SDL_WaitEvent" SDL_Event) (car e))))))

(define (sdl-push-event e)
  (zero? ((foreign-lambda int "SDL_PushEvent" SDL_Event) e)))

(define sdl-event-state (foreign-lambda int "SDL_EventState" unsigned-int integer))

;---------------------------------------------------------------------------

(define sdl-get-video-surface (foreign-lambda SDL_Surface "SDL_GetVideoSurface"))

(define (sdl-video-driver-name)
  (let ((bv (make-byte-vector 128 0)))
    (and ((foreign-lambda bool "SDL_VideoDriverName" byte-vector integer)
	  bv
	  (byte-vector-length bv))
	 (string-trim-right (byte-vector->string bv)
			    (integer->char 0)))))

(define sdl-set-video-mode (foreign-lambda SDL_Surface "SDL_SetVideoMode"
					   integer ; width
					   integer ; height
					   integer ; bpp
					   unsigned-integer ; flags
					   ))

(define (sdl-video-mode-ok w h bpp flags)
  (let ((result ((foreign-lambda integer "SDL_VideoModeOK"
				 integer integer
				 integer unsigned-integer)
		 w h bpp flags)))
    (and (not (zero? result))
	 result)))

(define (sdl-show-cursor . toggle)
  (if (null? toggle)
      ((foreign-lambda int "SDL_ShowCursor" int) -1)
      ((foreign-lambda int "SDL_ShowCursor" int) (if (car toggle) 1 0))))

(define sdl-map-rgb (foreign-lambda unsigned-integer "SDL_MapRGB"
				    SDL_PixelFormat
				    byte
				    byte
				    byte))

(define sdl-map-rgba (foreign-lambda unsigned-integer "SDL_MapRGBA"
				     SDL_PixelFormat
				     byte
				     byte
				     byte
				     byte))

;---------------------------------------------------------------------------

(define sdl-fill-rect (foreign-lambda int "SDL_FillRect" SDL_Surface SDL_Rect unsigned-integer))

(define sdl-flip (foreign-lambda int "SDL_Flip" SDL_Surface))

(define sdl-create-rgb-surface (foreign-lambda SDL_Surface "SDL_CreateRGBSurface"
					       unsigned-integer ; flags
					       integer ; width
					       integer ; height
					       integer ; depth
					       unsigned-integer ; rmask
					       unsigned-integer ; gmask
					       unsigned-integer ; bmask
					       unsigned-integer)) ; amask
(define sdl-free-surface (foreign-lambda void "SDL_FreeSurface" SDL_Surface))

(define sdl-blit-surface (foreign-lambda integer "SDL_BlitSurface"
					 SDL_Surface SDL_Rect ; src, srcrect
					 SDL_Surface SDL_Rect)) ; dst, dstrect

(define (sdl-with-clip-rect s r thunk)
  (let ((orig-clip-rect (make-sdl-rect 0 0 0 0)))
    (dynamic-wind
	(lambda ()
	  (sdl-get-clip-rect! s orig-clip-rect)
	  (sdl-set-clip-rect! s r))
	thunk
	(lambda ()
	  (sdl-set-clip-rect! s orig-clip-rect)))))

;---------------------------------------------------------------------------

(define-foreign-variable sizeof-sdl-color int "sizeof(SDL_Color)")

(define-record sdl-color buffer)

(let ((maker make-sdl-color))
  (set! make-sdl-color
	(lambda (r g b)
	  (let ((bv (make-byte-vector sizeof-sdl-color)))
	    (fill-sdl-color! (maker bv) r g b)))))

(define-record-printer (sdl-color s out)
  (for-each (lambda (x) (display x out))
	    (list "#<sdl-color "
		  (sdl-color-r s)" "
		  (sdl-color-g s)" "
		  (sdl-color-b s)">")))

(define (-sdl-unbox-color e)
  (let ((p (##sys#make-pointer)))
    (##core#inline "C_pointer_to_block" p (sdl-color-buffer e))
    p))

(define-foreign-type SDL_Color (pointer "SDL_Color")
  -sdl-unbox-color)

(define (fill-sdl-color! c r g b)
  ((foreign-lambda* void ((SDL_Color c) (byte r) (byte g) (byte b))
		    "c->r = r; c->g = g; c->b = b;")
   c r g b)
  c)

(define sdl-color-r (foreign-lambda* byte ((SDL_Color c)) "return(c->r);"))
(define sdl-color-g (foreign-lambda* byte ((SDL_Color c)) "return(c->g);"))
(define sdl-color-b (foreign-lambda* byte ((SDL_Color c)) "return(c->b);"))

;---------------------------------------------------------------------------

(define-record ttf-font pointer)

(define-record-printer (ttf-font f out)
  (for-each (lambda (x) (display x out))
	    (list "#<ttf-font "(ttf-font-pointer f)">")))

(define-foreign-type TTF_Font (pointer "TTF_Font")
  ttf-font-pointer
  make-ttf-font)

(define ttf-init (foreign-lambda integer "TTF_Init"))
(define ttf-was-init (foreign-lambda integer "TTF_WasInit"))
(define ttf-quit (foreign-lambda void "TTF_Quit"))

(define ttf-open-font (foreign-lambda TTF_Font "TTF_OpenFont" c-string integer))
(define ttf-open-font-index (foreign-lambda TTF_Font "TTF_OpenFontIndex" c-string integer long))
(define ttf-close-font (foreign-lambda void "TTF_CloseFont" TTF_Font))

(--sdl-flags "TTF_STYLE_NORMAL"
	     "TTF_STYLE_BOLD"
	     "TTF_STYLE_ITALIC"
	     "TTF_STYLE_UNDERLINE")

(define ttf-get-font-style (foreign-lambda integer "TTF_GetFontStyle" TTF_Font))
(define ttf-set-font-style (foreign-lambda void "TTF_SetFontStyle" TTF_Font integer))

(define ttf-font-height (foreign-lambda integer "TTF_FontHeight" TTF_Font))
(define ttf-font-ascent (foreign-lambda integer "TTF_FontAscent" TTF_Font))
(define ttf-font-descent (foreign-lambda integer "TTF_FontDescent" TTF_Font))
(define ttf-font-line-skip (foreign-lambda integer "TTF_FontLineSkip" TTF_Font))
(define ttf-font-faces (foreign-lambda long "TTF_FontFaces" TTF_Font))

(define ttf-font-face-is-fixed-width? (foreign-lambda bool "TTF_FontFaceIsFixedWidth" TTF_Font))
(define ttf-font-face-family-name (foreign-lambda c-string "TTF_FontFaceFamilyName" TTF_Font))
(define ttf-font-face-style-name (foreign-lambda c-string "TTF_FontFaceStyleName" TTF_Font))

(define ttf-render-text-solid (foreign-lambda* SDL_Surface ((TTF_Font font)
							    (c-string text)
							    (SDL_Color fg))
					       "return(TTF_RenderText_Solid(font,text,*fg));"))
(define ttf-render-utf8-solid (foreign-lambda* SDL_Surface ((TTF_Font font)
							    (c-string text)
							    (SDL_Color fg))
					       "return(TTF_RenderUTF8_Solid(font,text,*fg));"))

(define ttf-render-text-shaded
  (foreign-lambda* SDL_Surface ((TTF_Font font)
				(c-string text)
				(SDL_Color fg)
				(SDL_Color bg))
		   "return(TTF_RenderText_Shaded(font,text,*fg,*bg));"))

(define ttf-render-utf8-shaded
  (foreign-lambda* SDL_Surface ((TTF_Font font)
				(c-string text)
				(SDL_Color fg)
				(SDL_Color bg))
		   "return(TTF_RenderUTF8_Shaded(font,text,*fg,*bg));"))

(define ttf-render-text-blended (foreign-lambda* SDL_Surface ((TTF_Font font)
							      (c-string text)
							      (SDL_Color fg))
						 "return(TTF_RenderText_Blended(font,text,*fg));"))
(define ttf-render-utf8-blended (foreign-lambda* SDL_Surface ((TTF_Font font)
							      (c-string text)
							      (SDL_Color fg))
						 "return(TTF_RenderUTF8_Blended(font,text,*fg));"))

;---------------------------------------------------------------------------

(define img-load (foreign-lambda SDL_Surface "IMG_Load" c-string))

;---------------------------------------------------------------------------

(define rotozoom-surface (foreign-lambda SDL_Surface "rotozoomSurface"
					 SDL_Surface ; src
					 double ; angle
					 double ; zoom
					 bool)) ; smooth

(define zoom-surface (foreign-lambda SDL_Surface "zoomSurface"
				     SDL_Surface ; src
				     double ; zoomx
				     double ; zoomy
				     bool)) ; smooth
